/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2005 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootOverTime.cpp: implementation of the RootOverTime class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RootOverTime.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//empty constructure
RootOverTime::RootOverTime()
{
}

//empty destructor
RootOverTime::~RootOverTime()
{
}

// -------------------------------------------------------------------

//returns a RootInstance given the entered values
RootInstance * RootOverTime::GetRootInstance(int session, bool *still_live)
{
   *(still_live) = (m_still_alive)||((session - m_first_session + 1 < m_instances.size())&&(session >= m_first_session));
   if ((session - m_first_session < m_instances.size()) && (session - m_first_session >= 0))
   {
      return &m_instances.at(session - m_first_session);
   } //end if
  
   if((m_instances.size() > 0)&&(!m_still_alive)&&(session - m_first_session >= 0))
   {
      return &m_instances.at(m_instances.size()-1);
   } //end if

   return NULL;
}

// -------------------------------------------------------------------

//creates a new root instance given the entered session
RootInstance * RootOverTime::CreateRootInstance(int session)
{
   RootInstance temp_root;
   CPoint temp_point;

   //if this is the first session
   if(m_instances.size() == 0)
   {
      m_first_session = session;
      m_instances.push_back(temp_root);
      m_still_alive = true;

      return &m_instances.at(0);
   } //end if

   //if the root is still alive
   if(m_still_alive)
   {
      temp_root = m_instances.at(m_instances.size()-1);
      while (m_instances.size() <= (session - m_first_session))
      {
         m_instances.push_back(temp_root);
      } //end while

      return &m_instances.at(m_instances.size()-1);
   } //end if

  return NULL;
}

// -------------------------------------------------------------------

//sets a RootOverTime as being dead in the given session
void RootOverTime::DeclareDead(int session, bool dead, bool all_time)
{
   if(dead)
   {
      m_still_alive = false;
      if(all_time)
        m_instances.resize(0);
      else
        m_instances.resize(session-m_first_session+1);
   }
   else
   {
      m_still_alive = true;
      m_instances.resize(session+1);
      for (int i=1; i<m_instances.size(); i++)
      {
        m_instances.at(i).points = m_instances.at(i-1).points;
        m_instances.at(i).diameter = m_instances.at(i-1).diameter;
        m_instances.at(i).color = m_instances.at(i-1).color;
        m_instances.at(i).diameter_center = m_instances.at(i-1).diameter_center;
      }
   }
}

// -------------------------------------------------------------------

//returns the offset for a given session
CPoint RootOverTime::GetOffset(int session)
{
   CPoint temp;
   if((session - m_first_session >= 0)&&(session - m_first_session < m_instances.size())) 
   {
      return m_offset.at(session - m_first_session);
   } //end if

   // there isn't a root for this session, so offset is 0
   temp.x = 0;
   temp.y = 0;
   return temp;
} //end GetOffset

// -------------------------------------------------------------------

//sets the given offset for the given session
void RootOverTime::SetOffset(int session, CPoint offset)
{
   if((session - m_first_session >= 0)&&(session - m_first_session < m_instances.size())) 
   {
      m_offset.at(session - m_first_session) = offset;
   } //end if
} //end SetOffset

// -------------------------------------------------------------------

//returns the first session of a root
int RootOverTime::GetFirstSession() 
{
	return m_first_session + 1;  // zero-based indexing
}

// -------------------------------------------------------------------

//returns the size of a RootOverTime
int RootOverTime::GetNumInstances() 
{
	return m_instances.size();
}

// -------------------------------------------------------------------

//returns whether a root is dead or not
bool RootOverTime::GetStatus() 
{
   return m_still_alive;
}

// -------------------------------------------------------------------


void RootOverTime::SetRootInstance(int session, RootInstance root)
{
   if(session - m_first_session >= 0)
   {
      if((session - m_first_session) < m_instances.size())
      {
         m_instances.at(session - m_first_session) = root;
      }
      else if((session - m_first_session) == m_instances.size())
      {
         m_instances.resize(m_instances.size() + 1);
         m_instances.at(session - m_first_session) = root;
      }
   }
}