/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2005 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootflyDlg.h : header file
//

#if !defined(AFX_ROOTFLYDLG_H__28C31773_40C9_41D8_BB09_DB843E7D11A3__INCLUDED_)
#define AFX_ROOTFLYDLG_H__28C31773_40C9_41D8_BB09_DB843E7D11A3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma warning(disable:4786)

//#define IMAGE_X_ZERO 132
//#define IMAGE_Y_ZERO 31
//#define IMAGE_SCALE 2

#include "stdafx.h"
#include "Session_Array.h"
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
// CRootflyDlg dialog
enum Action
{
  NONE,
  ADD_LINE,
  ADD_DIAMETER,
  REGISTER
};

class CRootflyDlg : public CDialog
{
// Construction
public:
	CRootflyDlg(CWnd* pParent = NULL);	// standard constructor
  
// Dialog Data
	//{{AFX_DATA(CRootflyDlg)
	enum { IDD = IDD_ROOTFLY_DIALOG };
	CStatic	m_static_pixels_per_mm;
	CButton	m_button_extend_end;
	CButton	m_check_show_ids;
	CButton	m_button_all_dead;
	CButton	m_check_show_dead;
	CButton	m_copy_previous;
	CStatic	m_Transect_Pict_5;
	CStatic	m_Transect_Pict_4;
	CStatic	m_Transect_Pict_3;
	CStatic	m_Transect_Pict_2;
	CStatic	m_Transect_Pict_1;
	CStatic	m_Session_Pict_5;
	CStatic	m_Session_Pict_4;
	CStatic	m_Session_Pict_3;
	CStatic	m_Session_Pict_2;
	CStatic	m_Session_Pict_1;
	CEdit	m_edit_length;
	CEdit	m_edit_diameter;
	CComboBox	m_combo_state;
	CComboBox	m_combo_color;
	CEdit	m_edit_id;
	CButton	m_register_control;
	CEdit	m_edit_window;
	CEdit	m_edit_tube;
	CEdit	m_edit_transect;
	CEdit	m_edit_session;
	CSpinButtonCtrl	m_spin_window;
	CSpinButtonCtrl	m_spin_session;
	CSpinButtonCtrl	m_spin_tube;
	CSpinButtonCtrl	m_spin_transect;
	CButton	m_new_root_control;
	CButton	m_static_image;
	//}}AFX_DATA

   CString ExtractDirectory(CString filename);
   CString ExtractFile(CString filename);
//   CString ImageName(CString format,int tube,char transect,int window,int session);
   CString ImageName(CString directory, int tube, int window, int session);
   bool IsBitmap(CString fname);
   void UpdateTitleBar();

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CRootflyDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	HICON m_hIcon;

private:

   // action status vars
   Action m_action; // enum indicating the current state of the dialog
   int m_number_click; // indicate how many clicks
   bool m_addline; // indicate if the program goes into the add-line status.
   bool m_adddiameter; // indicate if the program goes into the add-diamter status.
   bool m_registering; // indicate if the program is in registering mode
   bool m_extend_end; //indicate if the program is adding a point to line end
   bool m_editing_point; // are we editing a point on a root
   bool m_moving_diameter; // are we moving the diameter
   CPoint m_moving_diameter_offset;  // if we are moving the diameter, this is the offset between the mouse position and the center (in image coordinates)
   bool m_resizing_diameter;
   bool m_have_set_default_image_format;

   // cursors
   HCURSOR m_cur_openhand;
   HCURSOR m_cur_closedhand;
   HCURSOR m_cur_uparrow;
   HCURSOR m_cur_sizeall;
   HCURSOR m_cur_sizens;

   CPoint m_image_topleft;
   double m_image_scale;

   // images and files
   blepo::ImgBgr m_image;
   CString m_filename;			// file name
   CString m_file_directory;
   CString m_image_filename;
   CString m_base_title;
   int m_order;

   //Mini Image variables
   blepo::ImgBgr m_Window_Pics[5];
   blepo::ImgBgr m_Session_Pics[5];

   // Variables for root related stuff
   RootflyData m_data;
   RootList *m_current_rootlist;
   CPoint m_curr_offset;
   int m_curr_tube, m_curr_transect, m_curr_window, m_curr_session;
   int m_selected_root_id;
   int m_selected_point_id;
   int m_highlighted_root_id;
   int m_millimeter;
   int m_ascii_add;

   bool m_open;

   // temp vars
   CPoint m_register_point;
   CPoint m_temp_offset;
   RootInstance m_temp_root;

   // jfisher: added 03-04-06
   RootInstance *context_temp_root;
   Session_Array m_session_array;


  // functions
  void AdjustPoint(CPoint &point);
  void ReAdjustPoint(CPoint &point);
  int IsOnRoot(CPoint point);
  int IsOnPoint(CPoint point);
  bool IsOnDiameter(CPoint point);
  bool IsOnDiameterPoint(CPoint point);
  void ReDraw(int mode=0, CPoint mouse_location = CPoint(0,0));
  bool ChangeLocation(int tube, int transect, int window, int session);
  void UpdateThumbnails(); // Draws the images of the next windows and sessions
  bool ScanDirectory();
  int DetermineClosestEnd(CPoint mouse, RootInstance *current);
  bool CheckForRoots(int curr_sess, int prev_sess, int tu, int tr, int wn);
  void UpdatePixelsPerMmDisplay();
  void EnableDisable(bool file_loaded);
  void EnableDisableRootControls(bool file_loaded);

	// Generated message map functions
	//{{AFX_MSG(CRootflyDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnLoadImages();
	afx_msg void OnNewRoot();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnSaveAs();
	afx_msg void OnOpen();
	afx_msg void OnExit();
	afx_msg void OnContextBlack();
	afx_msg void OnContextDarkBrown();
	afx_msg void OnContextLightBrown();
	afx_msg void OnContextWhite();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnSave();
	afx_msg void OnDeltaposSpinTube(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeltaposSpinSession(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeltaposSpinTransect(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeltaposSpinWindow(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnNew();
	afx_msg void OnButtonRegister();
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnClose();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	virtual void OnOK();
	afx_msg void OnKillfocusEditDiameter();
	afx_msg void OnKillfocusEditTube();
	afx_msg void OnKillfocusEditTransect();
	afx_msg void OnKillfocusEditWindow();
	afx_msg void OnKillfocusEditSession();
	afx_msg void OnSelchangeComboColor();
	afx_msg void OnSelchangeComboState();
	afx_msg void OnCopyPrevious();
	afx_msg void OnCheckShowDead();
	afx_msg void OnButtonAllDead();
	afx_msg void OnCheckShowIds();
	afx_msg void OnExport();
	afx_msg void OnCopySession();
	afx_msg void OnButtonExtendEnd();
	afx_msg void OnMmConversion();
  afx_msg void OnChangeDirectory();
	afx_msg void OnHelpAboutrootfly();
	afx_msg void OnEditNewroot();
	afx_msg void OnEditDeleteroot();
	afx_msg void OnEditExtendendpoint();
	afx_msg void OnEditRegister();
	afx_msg void OnEditCopyfromprevious();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_ROOTFLYDLG_H__28C31773_40C9_41D8_BB09_DB843E7D11A3__INCLUDED_)
